from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from daystrom.server.computer import Computer


class Transport(ABC):
    computer: Computer

    @abstractmethod
    async def start(self):
        pass

    @abstractmethod
    async def stop(self):
        pass
